@@ -3,6 +3,7 @@ use strict;
 use warnings;
 
 use Carmel;
+use Carmel::Runtime;
 use Carp ();
 use Carmel::Repository;
 use Config qw(%Config);
@@ -20,11 +21,11 @@ use Class::Tiny {
     perl_arch => sub { "$Config{version}-$Config{archname}" },
 };
 
-our $UseSystem = 0; # unit testing
-
 sub parse_options {
     my($self, $args) = @_;
 
+    return if $args->[0] && $args->[0] eq 'exec';
+
     my $cmd;
     my $parser = Getopt::Long::Parser->new(
         config => [ "no_ignore_case", "pass_through" ],
@@ -102,6 +103,8 @@ sub cmd_install {
     } else {
         $self->install_from_cpanfile(@args);
     }
+
+    $self->dump_bootstrap;
 }
 
 sub install_from_cpanfile {
@@ -166,31 +169,90 @@ sub install {
     my $dir = Path::Tiny->tempdir;
     local $ENV{PERL_CPANM_HOME} = $dir;
     local $ENV{PERL_CPANM_OPT};
-    system $^X, $self->fatscript, "--quiet", "--notest", "-L", $self->cache_dir, @args if @args;
+    system $^X, $self->fatscript,
+      ($self->verbose ? () : "--quiet"),
+      "--notest",
+      "-L", $self->cache_dir,
+      @args;
 
     for my $ent ($dir->child("latest-build")->children) {
-        next unless $ent->is_dir && $ent->child("blib/meta/install.json")->exists;
+        next unless $ent->is_dir && $ent->child("blib", "meta", "install.json")->exists;
         $self->repo->import_artifact($ent);
     }
 }
 
+sub dump_bootstrap {
+    my($self) = @_;
+
+    require B;
+    
+    my @artifacts;
+    $self->resolve(sub { push @artifacts, $_[0] });
+
+    my %modules;
+    for my $artifact (@artifacts) {
+        %modules = (%modules, %{$artifact->module_files});
+    }
+
+    my $file = Path::Tiny->new(".carmel/MyBootstrap.pm");
+    $file->parent->mkpath;
+    $file->spew(<<EOF);
+# This file serves dual purpose to load cached data in carmel exec setup phase
+# as well as on runtime to change \@INC
+package MyBootstrap;
+my(\%modules, \@inc, \@path);
+BEGIN {
+  \%modules = (
+    @{[ join ",\n    ", map { B::perlstring($_) . " => " . B::perlstring($modules{$_}) } keys %modules ]}
+  );
+  \@inc = (
+    @{[ join ",\n    ", map B::perlstring($_), map $_->sharedir_libs, @artifacts ]}
+  );
+  \@path = (
+    @{[ join ",\n    ", map B::perlstring($_), map $_->nonempty_paths, @artifacts ]}
+  );
+}
+
+use Carmel::Bootstrap;
+
+# for carmel exec setup
+Carmel::Bootstrap->environment(
+  modules=> \\\%modules,
+  inc  => \\\@inc,
+  path => \\\@path,
+  base => @{[ B::perlstring($file->parent->absolute) ]}
+);
+
+# for carmel exec runtime
+sub import {
+  Carmel::Bootstrap->bootstrap;
+}
+
+1;
+EOF
+}
+
+sub cmd_perl {
+    my $self = shift;
+    exec $^X, '-I.carmel', '-MCarmel::Bootstrap', @_;
+}
+
 sub cmd_export {
     my($self) = @_;
-    my %env = $self->env;
-    print "export PATH=$env{PATH} PERL5LIB=$env{PERL5LIB}\n";
+    my %env = Carmel::Runtime->new->env;
+    print "export ", join(" ", map qq($_="$env{$_}"), keys %env), "\n";
 }
 
 sub cmd_env {
     my($self) = @_;
-    my %env = $self->env;
-    print "PATH=$env{PATH}\nPERL5LIB=$env{PERL5LIB}\n";
+    my %env = Carmel::Runtime->new->env;
+    print join "", map qq($_="$env{$_}"\n), keys %env;
 }
 
+# TODO remove. just here for testing
 sub cmd_exec {
     my($self, @args) = @_;
-    my %env = $self->env;
-    %ENV = (%ENV, %env);
-    $UseSystem ? system(@args) : exec @args;
+    Carmel::Runtime->new->execute(@args);
 }
 
 sub cmd_find {
@@ -265,7 +327,7 @@ sub write_index {
 
     $self->resolve(sub {
         my $artifact = shift;
-        while (my($pkg, $data) = each %{$artifact->install->{provides}}) {
+        while (my($pkg, $data) = each %{$artifact->provides}) {
             my $package = Carton::Package->new($pkg, $data->{version} || 'undef', $artifact->install->{pathname});
             $index->add_package($package);
         }
@@ -343,11 +405,11 @@ sub resolve_recursive {
         my $want_version = $root_reqs->requirements_for_module($module);
         next if $self->is_core($module, $want_version);
 
-        warn "Resolving $module for $want_version\n" if $self->verbose;
+        warn "$depth: Resolving $module ($want_version)\n" if $self->verbose;
 
         # FIXME there's a chance different version of the same module can be loaded here
         if (my $artifact = $self->repo->find($module, $want_version)) {
-            warn "-> ", $artifact->path, "\n" if $self->verbose;
+            warn sprintf "   %s (%s) in %s\n", $module, $artifact->version_for($module), $artifact->path if $self->verbose;
             next if $seen->{$artifact->path}++;
             $cb->($artifact, $depth);
 
@@ -370,21 +432,4 @@ sub resolve {
     $self->resolve_recursive($self->requirements, $self->requirements->clone, {}, $cb, $missing_cb, 0);
 }
 
-sub env {
-    my($self) = @_;
-
-    my @artifacts;
-    $self->resolve(sub { push @artifacts, $_[0] });
-    return (
-        _join(PATH => map $_->paths, @artifacts),
-        _join(PERL5LIB => map $_->libs, @artifacts),
-    );
-}
-
-sub _join {
-    my($env, @list) = @_;
-    push @list, $ENV{$env} if $ENV{$env};
-    return ($env => join(":", @list));
-}
-
 1;
